#include "util.h"


/* ------------------------------------------------------------------- */
/* Methods for reading input data from a file contatining the instance  */


void read_problem_size(char name[100] ) {
  /* Read the dimension of the instance */    
  int i;
  char *num;
  char *filename;
  setbuf(stdout,NULL);
  
  filename=strdup(basename(name));
  i=strspn( filename, "wt" );
  num=strtok(filename+i,".");
  instance.number_jobs=atoi(num);
  
  if( instance.number_jobs <= 0) {
    fprintf(stderr, "error reading number of jobs\n");
    exit(1);	
  }
  VERBOSE ( printf("number_jobs = %ld\n",instance.number_jobs); )
    }


void read_instances( char input_file_name[100] ) {
  /* Read the job data of the instance */
  
  int i=0;
  long int 	j;
  FILE    *inputfile;
  
  inputfile = fopen(input_file_name, "r");
  assert(inputfile != NULL);
  

  for ( j = 0 ; j < instance.number_jobs ; j++ ) {
    instance.release_date[j] = 0; /* there is no release date for these instances */
  } 
  for ( j = 0 ; j < instance.number_jobs ; j++ ) {
    if( fscanf(inputfile, "%ld", &(instance.proc_time[j])) <= 0) {
      fprintf(stderr, "error reading (%ld, %ld) processing time in data file\n", j, instance.proc_time[j]);
      exit(1);	
    } 
  }
  for ( j = 0 ; j < instance.number_jobs ; j++ ) {
    if( fscanf(inputfile, "%ld", &(instance.weight[j])) <= 0) {
      fprintf(stderr, "error reading (%ld, %ld) weight in data file\n", j, instance.weight[j]);
      exit(1);	
    }
  }
  for ( j = 0 ; j < instance.number_jobs ; j++ ) {
    if( fscanf(inputfile, "%ld", &(instance.due_date[j])) <= 0) {
      fprintf(stderr, "error reading %ld deadline value in data file\n", instance.due_date[j]);
      exit(1);	
    } 
  }
  close(inputfile);
}     



/* ------------------------------------------------------------------- */
/* Methods for computing the cost of a solution */

long int evaluate() {

  /* Computes the evalaution function value of a sequence for the SMTWTP
     from scratch */

  long int  j;
  long int  f_value = 0;
  long int  start_time_job, end_time_job, job, prec_job;

  /* first compute the completion times of all jobs */

  /* initialize for the first job */
  job = sequence.job_at_pos[0];

  start_time_job = instance.release_date[job];
  end_time_job = start_time_job + instance.proc_time[job];

  sequence.start_time_job[job] = start_time_job;
  sequence.completion_time_job[job] = end_time_job;

sequence.tardiness_job[job] = MAX(end_time_job-instance.due_date[job],0);
  /* now the remaining jobs */
  for ( j = 1 ; j < instance.number_jobs ; j++ ) {
    job = sequence.job_at_pos[j];
    prec_job = sequence.job_at_pos[j-1];

    start_time_job = MAX(instance.release_date[job],sequence.completion_time_job[prec_job]);
    sequence.start_time_job[job] = start_time_job;

    end_time_job = start_time_job + instance.proc_time[job];
    sequence.completion_time_job[job] = end_time_job;
    sequence.tardiness_job[job] = MAX(end_time_job-instance.due_date[job],0);
  }

  /* Now compute the objective function value */
  for ( j = 0 ; j < instance.number_jobs ; j++ ) 
    f_value += instance.weight[j] * sequence.tardiness_job[j];

  sequence.value=f_value;

  return( f_value );
}



/* ------------------------------------------------------------------- */
/* Methods for printing solutions */


void print_sequence(long int k) {
  long int i;
  
  printf("Objective function value = %ld\n", sequence.value);
  for( i = 1 ; i <= instance.number_jobs ; i++ ) {
    printf("%ld ", sequence.job_at_pos[i-1]); 
    if ( !(i % 20) )
      printf("\n");
  }
 printf("\n");
}


void print_completion_times( ) {
  long int i;
  
  printf("Objective function value = %ld\n", sequence.value);
  for( i = 1 ; i <= instance.number_jobs ; i++ ) {
    printf("%ld ", sequence.completion_time_job[sequence.job_at_pos[i-1]]); 
    if ( !(i % 20) )
      printf("\n");
  }
 printf("\n");
}


/* ------------------------------------------------------------------- */
/* Methods for constructing initial solutions */

void construct_sequence_random( ) {
  int i;

  int random_vector[instance.number_jobs];
  for (i=0;i<instance.number_jobs;i++)
    random_vector[i]=i;
  VERBOSE ( printf("Construct random solution\n"); )
    shuffle(random_vector,instance.number_jobs);
  for( i = 0 ; i < instance.number_jobs ; i++ ) {
    sequence.job_at_pos[i] = random_vector[i];
    sequence.pos_of_job[random_vector[i]] = i;
  }
  sequence.value = evaluate();
}


void construct_sequence_canonical() {
  long int j;
  
  VERBOSE ( printf("Build canonical sequence\n"); )
    for( j = 0 ; j < instance.number_jobs ; j++ ) {
      sequence.job_at_pos[j] = j;
      sequence.pos_of_job[j] = j;
    }
  sequence.value = evaluate();
}
